package wrox.webservices.UDDIImpl;

import java.net.*;
import java.util.*;
import java.io.*;

import com.ibm.uddi.*;
import com.ibm.uddi.client.*;
import com.ibm.uddi.response.*;
import com.ibm.uddi.datatype.business.*; 

import org.apache.soap.SOAPException;
import org.apache.soap.transport.http.SOAPHTTPConnection;

/**
 * The DeleteBusiness class the business listing for a company.
 * Only the publisher of a businessEntity can remove it, and
 * this can only be done at the node where it was published.
 */
public class DeleteBusiness {

  /* Name of the business */
  private final static String BUSINESS_NAME        = "Crashes R'Us Airlines";

  /* The username and password to interact with the UDDI registry */
  private final static String USERNAME = "wrox-chapter8";
  private final static String PASSWORD = "secret";

  public static void main (String args[]) {

    /* Settings for HTTPS, SSL support */
    System.setProperty ("java.protocol.handler.pkgs", 
                        "com.ibm.net.ssl.internal.www.protocol");
    java.security.Security.addProvider (new com.ibm.jsse.JSSEProvider ());

    /* Construct a UDDIProxy object- this represents a UDDI server 
     * and the actions that can be invoked against it.
     */
    UDDIProxy uddiProxy = new UDDIProxy ();
    /* Setting for the proxy server- in case you are behind one */
    /*
     SOAPHTTPConnection soapHTTPConnection = new SOAPHTTPConnection ();
     soapHTTPConnection.setUserName ("proxy-username");
     soapHTTPConnection.setPassword ("proxy-password");
     soapHTTPConnection.setProxyHost ("proxyhostname");
     soapHTTPConnection.setProxyPort (8080);
     uddiProxy.setTransport (soapHTTPConnection);
    */

    /* Set the inquiry and publish URLs to the Test registry node at IBM */
    try {
      uddiProxy.setInquiryURL (UDDIOperatorURLs.IBM_INQUIRY_URL_TEST);
      uddiProxy.setPublishURL (UDDIOperatorURLs.IBM_PUBLISH_URL_TEST);
      System.out.println ("Inquiry URL: "
                          + UDDIOperatorURLs.IBM_INQUIRY_URL_TEST + "\n");
      System.out.println ("Publishing URL: "
                          + UDDIOperatorURLs.IBM_PUBLISH_URL_TEST + "\n");
    } catch (MalformedURLException e) {
      System.err.println ("Malformed URLs for the UDDI registry ["
                          + UDDIOperatorURLs.IBM_PUBLISH_URL_TEST + ", "
                          + UDDIOperatorURLs.IBM_INQUIRY_URL_TEST + "]");
      e.printStackTrace ();
      System.exit (1); /* fatal error */
    }

    /* Get an authentication token */
    AuthToken authToken = null;
    /* The Disposition report contains the return status from the call */
    DispositionReport dispositionReport = null;

    try {
      authToken = uddiProxy.get_authToken(USERNAME, PASSWORD);
    } catch (UDDIException e) {
      System.err.println ("UDDI error getting authentication token: [" 
                          + e.getFaultCode() + "]" + e.getFaultString()
                         );
      dispositionReport = e.getDispositionReport ();
      System.err.println ("More error information : " 
                          + dispositionReport.getErrCode() + ": "
                          + dispositionReport.getErrInfoText());
      System.exit (1); /* fatal error */
    } catch (SOAPException e) {
      System.err.println ("SOAP error getting authentication token " 
                          + e.getFaultCode() + ": " + e.getMessage());
      System.exit (1); /* fatal error */

    }

    /* Get all business entities and tModel published by us */
    RegisteredInfo registeredInfo = new RegisteredInfo ();
    try {
      System.out.println (
      "Getting all business entities and tModels published by us");
      registeredInfo = uddiProxy.get_registeredInfo 
                          (authToken.getAuthInfoString());
     } catch (UDDIException e) {
        System.err.println 
           ("UDDI error getting registered info: [" 
            + e.getFaultCode() + "]" + e.getFaultString());
        dispositionReport = e.getDispositionReport ();
        System.err.println ("More error information : " 
                          + dispositionReport.getErrCode() + ": "
                          + dispositionReport.getErrInfoText());
     } catch (SOAPException e) {
        System.err.println ("SOAP error getting registered info: " 
                            + e.getFaultCode() + ": " + e.getMessage());
     }

    Vector businessInfoVector = registeredInfo.getBusinessInfos().getBusinessInfoVector();
    Vector tModelInfoVector = registeredInfo.getTModelInfos().getTModelInfoVector();

    System.out.println ("Found " + businessInfoVector.size()
                        + " business entities and "
                        + tModelInfoVector.size() 
                        + " tModels published by us"
                       );

    /* Delete the business entity */
    char reply = 'n'; 
    if (!businessInfoVector.isEmpty()) {
       System.out.println ("Remove business entities? [yn]: ");
       InputStreamReader inputReader = new InputStreamReader (System.in);
       try {
         reply = (char) inputReader.read();
       } catch (Exception e) {/*ignore*/}
    } 

    if (reply == 'y') {
      /* Assuming only one business entity; as that's all you are
         allowed to publish */
      BusinessInfo businessInfo = (BusinessInfo) businessInfoVector.elementAt(0);
      String businessKey = businessInfo.getBusinessKey ();
    
      try {
        System.out.println ("\nAttempting to delete business with businessKey ["
                     + businessKey + "]");
        dispositionReport = uddiProxy.delete_business (
                                   authToken.getAuthInfoString(),
                                   businessKey);
      } catch (UDDIException e) {
        System.err.println 
            ("UDDI related error while deleting business entity: [" 
            + e.getFaultCode() + "]" + e.getFaultString());
        dispositionReport = e.getDispositionReport ();
        System.err.println ("More error information : " 
                          + dispositionReport.getErrCode() + ": "
                          + dispositionReport.getErrInfoText());
      } catch (SOAPException e) {
        System.err.println ("SOAP related error deleting business entity: " 
                            + e.getFaultCode() + ": " + e.getMessage());
      }

      if (dispositionReport.success ()) {
        System.out.println ("Successfully deleted businesEntity for "
                           + BUSINESS_NAME + "[" + businessKey + "]");
      } else {
        System.err.println ("Error in deleting businesEntity for "
               + BUSINESS_NAME + "[" + businessKey + "]:"
               + "Errno (" + dispositionReport.getErrno() + ") "
               + "ErrCode (" + dispositionReport.getErrCode() + ") "
               + "ErrText (" + dispositionReport.getErrInfoText() + ") ");
      }
    } /* remove business entity */
  } /* end main */
}
