package wrox.webservices.chapter8;

import java.net.*;
import java.util.*;
import java.io.*;

import com.ibm.uddi.*;
import com.ibm.uddi.client.*;
import com.ibm.uddi.response.*;
import com.ibm.uddi.datatype.business.*; 

/**
 * The DeleteBusiness class the business listing for a company.
 * Only the publisher of a businessEntity can remove it, and
 * this can only be done at the node where it was published.
 */
public class DeleteBusiness {

  /* The username and password to interact with the UDDI registry */
  private static String USERNAME = "wrox-chapter7b-ex2";
  private static String PASSWORD = "secret";

  /* The business name */
  private static String BUSINESS_NAME = "Crashes R'Us Airlines";

  public static void main (String args[]) {

    /* Settings for HTTPS, SSL support */
    System.setProperty ("java.protocol.handler.pkgs", 
                        "com.ibm.net.ssl.internal.www.protocol");
    java.security.Security.addProvider (new com.ibm.jsse.JSSEProvider ());

    /* Construct a UDDIProxy object- this represents a UDDI server 
     * and the actions that can be invoked against it.
     */
    UDDIProxy uddiProxy = new UDDIProxy ();

    /* Set the inquiry and publish URLs to the Test registry node at IBM */
    try {
      uddiProxy.setInquiryURL (Constants.IBM_INQUIRY_URL_TEST);
      uddiProxy.setPublishURL (Constants.IBM_PUBLISH_URL_TEST);
      System.out.println ("Inquiry URL: "
                          + Constants.IBM_INQUIRY_URL_TEST + "\n");
      System.out.println ("Publishing URL: "
                          + Constants.IBM_PUBLISH_URL_TEST + "\n");
    } catch (MalformedURLException e) {
      System.err.println ("Malformed URLs for the UDDI registry ["
                          + Constants.IBM_PUBLISH_URL_TEST + ", "
                          + Constants.IBM_INQUIRY_URL_TEST + "]");
      e.printStackTrace ();
      System.exit (1); /* fatal error */
    }

    /* Get an authorization token */
    AuthToken authToken = null;
    try {
      authToken = uddiProxy.get_authToken(USERNAME, PASSWORD);
    } catch (Exception e) { /* Either UDDI or SOAP Exception */
      System.err.println ("Error getting authorization token: " 
                          + e.getMessage());
      e.printStackTrace ();
      System.exit (1); /* fatal error */
    }

    /* The Disposition report contains the return status from the call */
    DispositionReport dispositionReport = null;

    /* In case we know the business key, we can delete the businessEntity
     * using the key:
     *   dispositionReport = uddiProxy.delete_business (
     *                              authToken.getAuthInfoString(),
     *                              "BFFB4080-6042-11D5-A58E-0004AC49CC1E");
     * Alternatively, we can search based on the business name, and then
     * delete, as shown below:
     */

     /* Get the businessEntity's business key 
      * - find the business by name (or other search criteria)
      * - get its business key
      * - attempts to delete a business not created by the user will fail
      */
     System.out.println ("Attempting to delete the businessEntity for "
                         + BUSINESS_NAME);
     BusinessList businessList = null;
     try {
       businessList = uddiProxy.find_business(BUSINESS_NAME, 
                                              null, 0);
     } catch (Exception e) { /* Either UDDI or SOAP Exception */
       System.err.println ("Error finding business " + BUSINESS_NAME + ": " 
                           + e.getMessage());
       e.printStackTrace ();
       System.exit (1); /* fatal error */
     }

     /* The search returns all partial matches for the business name. Here
      * we make an (not a very wise) assumption that we get just one business
      * - the one that we wish to delete.
      */
     Vector businessInfoVector = 
        businessList.getBusinessInfos().getBusinessInfoVector();

     if (businessInfoVector.isEmpty ()) {
       System.out.println ("No business matching " + BUSINESS_NAME 
                           + " found. Sorry!");
       System.exit (0);
     }

     /* Take the first business found */
     BusinessInfo businessInfo = (BusinessInfo) businessInfoVector.elementAt(0);
     String businessKey = businessInfo.getBusinessKey ();

     try {
       dispositionReport = uddiProxy.delete_business (
                                   authToken.getAuthInfoString(),
                                   businessKey);
     } catch (Exception e) { /* SOAP or UDDI exception */ 
       System.err.println ("Exception deleting business "
                           + e.getMessage());
       e.printStackTrace ();
       System.exit (1); /* fatal error */
     }

     if (dispositionReport.success ()) {
       System.out.println ("Successfully deleted businesEntity for "
                           + BUSINESS_NAME + "[" + businessKey + "]");
     } else {
       System.err.println ("Error in deleting businesEntity for "
               + BUSINESS_NAME + "[" + businessKey + "]:"
               + "Errno (" + dispositionReport.getErrno() + ") "
               + "ErrCode (" + dispositionReport.getErrCode() + ") "
               + "ErrText (" + dispositionReport.getErrInfoText() + ") "
		   );
     }
  }
}
