package wrox.webservices.chapter8;

import java.net.*;
import java.util.*;
import java.io.*;

import com.ibm.uddi.*;
import com.ibm.uddi.client.*;
import com.ibm.uddi.response.*;
import com.ibm.uddi.datatype.binding.*;
import com.ibm.uddi.datatype.business.*; 
import com.ibm.uddi.datatype.tmodel.*;
import com.ibm.uddi.datatype.service.*;
import com.ibm.uddi.util.*;

import org.apache.soap.SOAPException;

/**
 * The SaveService class publishes businesServices(s) for
 * our fictitious airline
 */
public class SaveService {

  /* The username and password to interact with the UDDI registry */
  private static String USERNAME = "yourname";
  private static String PASSWORD = "yourpassword";

  /* Service related constants */
  private static String BUSINESS_NAME = "Crashes R'Us Airlines";
  private static String TMODEL_NAME   = "fare_service_tmodel";
  private static String SERVICE_NAME  = "fare_service";
  private static String SERVICE_URL   = "http://127.0.0.1:8080/soap/servlet/rpcrouter";
  private static String SERVICE_DESC  = "SOAP service for fare quotes";

  public static void main (String args[]) {

    /* Settings for HTTPS, SSL support */
    System.setProperty ("java.protocol.handler.pkgs", 
                        "com.ibm.net.ssl.internal.www.protocol");
    java.security.Security.addProvider (new com.ibm.jsse.JSSEProvider ());

    /* Construct a UDDIProxy object- this represents a UDDI server 
     * and the actions that can be invoked against it.
     */
    UDDIProxy uddiProxy = new UDDIProxy ();

    /* Set the publish URL to the Test registry node at IBM */
    try {
      uddiProxy.setInquiryURL (Constants.IBM_INQUIRY_URL_TEST);
      uddiProxy.setPublishURL (Constants.IBM_PUBLISH_URL_TEST);
      System.out.println ("Inquiry URL: "
                          + Constants.IBM_INQUIRY_URL_TEST + "\n");
      System.out.println ("Publishing URL: "
                          + Constants.IBM_PUBLISH_URL_TEST + "\n");
    } catch (MalformedURLException e) {
      System.err.println ("Malformed URLs for the UDDI registry ["
                          + Constants.IBM_PUBLISH_URL_TEST + ", "
                          + Constants.IBM_INQUIRY_URL_TEST + "]");
      e.printStackTrace ();
    }

    /* Get an authorization token */
    AuthToken authToken = null;
    try {
      authToken = uddiProxy.get_authToken(USERNAME, PASSWORD);
    } catch (Exception e) { /* Either UDDI or SOAP Exception */
      System.err.println ("Error getting authorization token: " 
                          + e.getMessage());
      e.printStackTrace ();
      System.exit (1); /* fatal error */
    }

    /* Get the tModel key. We are currently searching by tModel name-
     * we could instead use a well known (and unique) tModel key.
     */
    TModelList tModelList = null;
    try {
      tModelList = uddiProxy.find_tModel (TMODEL_NAME, null, 0);
     } catch (Exception e) { /* Either UDDI or SOAP Exception */
      System.err.println ("Error finding tModel : " 
                          + e.getMessage());
      e.printStackTrace ();
      System.exit (1); /* fatal error */
    }

    Vector tModelVector = tModelList.getTModelInfos().getTModelInfoVector();
    /* Assuming that there is one and only one tModel matching TMODEL_NAME */
    String tModelKey = ((TModelInfo)tModelVector.elementAt(0)).getTModelKey ();
    System.out.println ("Found tModel " + TMODEL_NAME 
                        + " with tModel key " + tModelKey);


    /* Get the businessKey for the business we want to publish the 
     * service in- again, instead of using the company name, we could 
     * use a well known (and unique) businessKey
     */
    BusinessList businessList = null;
    try {
      businessList = uddiProxy.find_business (BUSINESS_NAME, null, 0);
    } catch (Exception e) { /* Either UDDI or SOAP Exception */
      System.err.println ("Error finding business : " 
                          + e.getMessage());
      e.printStackTrace ();
      System.exit (1); /* fatal error */
    }

    Vector businessVector = businessList.getBusinessInfos().getBusinessInfoVector();
    /* Assuming that there is one and only one businessEntity matching 
     * BUSINESS_NAME */
    String businessKey = ((BusinessInfo)businessVector.elementAt(0)).getBusinessKey ();

    System.out.println ("Found business " + BUSINESS_NAME 
                        + " with Business key " + businessKey);

    /* Construct the bindingTemplate structure */
    TModelInstanceInfo tModelInstanceInfo = new TModelInstanceInfo (tModelKey);
    Vector tModelInstanceInfoVector = new Vector ();
    tModelInstanceInfoVector.addElement (tModelInstanceInfo);
    TModelInstanceDetails tModelInstanceDetails = new TModelInstanceDetails();
    tModelInstanceDetails.setTModelInstanceInfoVector (tModelInstanceInfoVector);
    BindingTemplate bindingTemplate = new BindingTemplate ();
    bindingTemplate.setTModelInstanceDetails (tModelInstanceDetails);
    bindingTemplate.setAccessPoint (new AccessPoint (SERVICE_URL, "http"));
    bindingTemplate.setDefaultDescriptionString (SERVICE_DESC);

    /* Construct the businessService structure */
    Vector bindingTemplateVector = new Vector ();
    bindingTemplateVector.addElement (bindingTemplate);
    BindingTemplates bindingTemplates = new BindingTemplates ();
    bindingTemplates.setBindingTemplateVector (bindingTemplateVector);

    BusinessService businessService = new BusinessService ("",
                                       SERVICE_NAME, bindingTemplates);
    businessService.setBusinessKey (businessKey);

    /* Publish the service information */
    Vector businessServices = new Vector ();
    businessServices.addElement (businessService);
    ServiceDetail serviceDetail = null;
    System.out.println ("Attempting to publish businessService " 
                        + SERVICE_NAME);
    try {
      serviceDetail = uddiProxy.save_service (authToken.getAuthInfoString(), 
                                              businessServices);
    } catch (UDDIException e) {
      System.err.println ("UDDI related error while saving business service: [" 
                          + e.getFaultCode() + "]" + e.getFaultString()
			 );
      DispositionReport dispositionReport = e.getDispositionReport ();
      System.err.println ("More error information : " 
                          + dispositionReport.getErrCode() + ": "
			  + dispositionReport.getErrInfoText());
      System.exit (1); /* fatal error */
    } catch (SOAPException e) {
      System.err.println ("SOAP related error saving business service: " 
                          + e.getFaultCode() + ": " + e.getMessage());
      System.exit (1); /* fatal error */

    }

    /* Get the service key assigned for our service */
    Vector businessServiceVector = serviceDetail.getBusinessServiceVector();

    BusinessService thisBusinessService = (BusinessService)
                                          (businessServiceVector.elementAt(0));
    String serviceKey = thisBusinessService.getServiceKey ();

    System.out.println ("The " + SERVICE_NAME + " service has"
                   + " been successfully updated  in the UDDI registry in"
		   + " the company " + BUSINESS_NAME 
		   + " and assigned the Service Key: " + serviceKey);

  }
}
