package wrox.webservices.chapter8;

import java.net.*;
import java.util.*;
import java.io.*;

import com.ibm.uddi.*;
import com.ibm.uddi.client.*;
import com.ibm.uddi.response.*;
import com.ibm.uddi.datatype.*;
import com.ibm.uddi.datatype.tmodel.*;
import com.ibm.uddi.datatype.business.*; 
import com.ibm.uddi.util.*;

/**
 * The SaveTModel class publishes a TModel description. 
 * A tModel acts as a unique signature for a service. Typically
 * standards bodies would publish tModels and individual companies
 * would write implementations conforming to it.
 */
public class SaveTModel {

  /* The username and password to interact with the UDDI registry */
  private static String USERNAME = "yourname";
  private static String PASSWORD = "yourpassword";

  /* The tModel information */
  private static String TMODEL_NAME = "fare_service_tmodel";
  private static String TMODEL_DESC = 
    "The fare service provides a programmatic API for getting airline fare information";
  private static String WSDL_URL    = 
    "http://127.0.0.1:8080/fare_service/fare_service.wsdl";

  /* Helper method for constructing a vector with one element in it */
  private static Vector constructVector (Object object) {
    Vector vector = new Vector ();
    vector.addElement (object);
    return vector;
  }

  public static void main (String args[]) {

    /* Settings for HTTPS, SSL support */
    System.setProperty ("java.protocol.handler.pkgs", 
                        "com.ibm.net.ssl.internal.www.protocol");
    java.security.Security.addProvider (new com.ibm.jsse.JSSEProvider ());

    /* Construct a UDDIProxy object- this represents a UDDI server 
     * and the actions that can be invoked against it.
     */
    UDDIProxy uddiProxy = new UDDIProxy ();

    /* Set the publish URL to the Test registry node at IBM */
    try {
      uddiProxy.setPublishURL (Constants.IBM_PUBLISH_URL_TEST);
      System.out.println ("Publishing tModel to " 
                          + Constants.IBM_PUBLISH_URL_TEST);
    } catch (MalformedURLException e) {
      System.err.println ("Malformed publish URL for the UDDI registry ["
                          + Constants.IBM_PUBLISH_URL_TEST + "]");
      e.printStackTrace ();
      System.exit (1); /* fatal error */
    }

    /* Get an authorization token */
    AuthToken authToken = null;
    try {
      authToken = uddiProxy.get_authToken(USERNAME, PASSWORD);
    } catch (Exception e) { /* Either UDDI or SOAP Exception */
      System.err.println ("Error getting authorization token: " 
                          + e.getMessage());
      e.printStackTrace ();
      System.exit (1); /* fatal error */
    }

    /* Construct the tModel - the null tModel key should cause the UDDI 
     * registry to assign a (unique) key for the tModel.
     */
    TModel tModel = new TModel (null, TMODEL_NAME);
    tModel.setDefaultDescriptionString (TMODEL_DESC);

    OverviewDoc overviewDoc = new OverviewDoc ();
    overviewDoc.setOverviewURL (new OverviewURL (WSDL_URL));
    overviewDoc.setDefaultDescriptionString ("WSDL Document");
    tModel.setOverviewDoc (overviewDoc);

    /* Publish the tModel in the registry */
    TModelDetail tModelDetail = null;
    try {
      tModelDetail = uddiProxy.save_tModel (authToken.getAuthInfoString (),
                                            constructVector (tModel));
    } catch (Exception e) { /* Either UDDI or SOAP Exception */
      System.err.println ("Error publishing tModel: " 
                         + e.getMessage());
      e.printStackTrace ();
      System.exit (1); /* fatal error */
   }

   /* Get the tModel key assigned for this tModel */
    Vector tModelVector = tModelDetail.getTModelVector();

   /* Since we saved only one tModel, the resulting tModel
    * detail should have only one tModel in it.
    */
    TModel thisTModel = (TModel) (tModelVector.elementAt(0));
    String tModelKey = thisTModel.getTModelKey ();

    System.out.println ("The " + TMODEL_NAME 
                      + " tModel has been successfully updated "
                      + " in the UDDI registry and assigned the tModel Key: "
                      + tModelKey);
  }
}
