package wrox.webservices.chapter8;

import java.net.*;
import java.text.*;
import java.util.*;
import java.io.*;

import com.ibm.uddi.*;
import com.ibm.uddi.client.*;
import com.ibm.uddi.datatype.binding.*; 
import com.ibm.uddi.datatype.business.*; 
import com.ibm.uddi.datatype.service.*;
import com.ibm.uddi.datatype.tmodel.*;
import com.ibm.uddi.response.*;
import com.ibm.uddi.util.*;

/**
 * UDDIClient class uses the UDDI4J Inquiry API to navigate
 * the UDDI registry. This example shows how a vendor company
 * tries to get information about buying products from IBM.
 */
public class UDDIClient {

  private static String BUSINESS_NAME = "IBM Corporation";
  private static String SERVICE_NAME  = 
  "Buy from IBM using business-to-business e-commerce applications";

  public static void main (String args[]) {

    /* Construct a UDDIProxy object- this represents a UDDI server 
     * and the actions that can be invoked against it.
     */
    UDDIProxy uddiProxy = new UDDIProxy ();

    /* Select which UDDI node to send the query to */
    if (args.length < 1) {
       System.out.println ("usage: UDDIClient <IBM_PROD|MS_PROD>");
       System.exit (1);
    }

    String inquiryUrl = null;
    if (args[0].equals ("IBM_PROD")) {
      inquiryUrl = Constants.IBM_INQUIRY_URL_PRODUCTION;
      System.out.println ("Inquiry URL set to IBM UDDI node [" 
                          + inquiryUrl + "]");
    } else { /* MS_PROD */
      inquiryUrl = Constants.MICROSOFT_INQUIRY_URL_PRODUCTION;
      System.out.println ("Inquiry URL set to Microsoft UDDI node [" 
                          + inquiryUrl + "]");
    }

    try {
      uddiProxy.setInquiryURL (inquiryUrl);
    } catch (MalformedURLException e) {
      System.err.println ("Malformed inquiry URL [" + inquiryUrl + "]");
      System.exit (1); /* fatal error */
    }

    /* Find all businesses matching the partial name "IBM" */
    BusinessList businessList = null;
    try {
        businessList = uddiProxy.find_business ("IBM", null, 0);
    } catch (Exception e) {
      System.err.println ("Exception finding business: " + e.getMessage ());
      System.exit (1); /* fatal error */
    }

    Vector businessInfoVector = 
       businessList.getBusinessInfos().getBusinessInfoVector();

    System.out.println ("Found " + businessInfoVector.size()
                        + " businesses matching the name IBM. These are: \n");

    String businessKey  = null;
    for (int i = 0; i < businessInfoVector.size(); i++) {
      String businessName = ((BusinessInfo)businessInfoVector.elementAt(i)).getNameString();
      System.out.println ("[" + i + "] " + businessName);
      if (businessName.equals (BUSINESS_NAME))
        businessKey = ((BusinessInfo)businessInfoVector.elementAt(i)).getBusinessKey();
    }

    System.out.println ("The business key for " + BUSINESS_NAME + " is " +
                        businessKey);

    /* Drill down for more information about the business */
    Vector businessKeyVector = new Vector ();
    businessKeyVector.addElement (businessKey);

    BusinessDetail businessDetail = null;
    try {
        businessDetail = uddiProxy.get_businessDetail (businessKeyVector);
    } catch (Exception e) {
      System.err.println ("Exception finding business detail: " + e.getMessage ());
      System.exit (1); /* fatal error */
    }

    /* Assuming one business entity */
    BusinessEntity businessEntity = (BusinessEntity) businessDetail.getBusinessEntityVector().elementAt(0);
    Vector businessServiceVector = businessEntity.getBusinessServices().getBusinessServiceVector();

    System.out.println ("Found " + businessServiceVector.size()
                        + " services provided the business " + BUSINESS_NAME
			+ ". These are: \n");

    BusinessService businessService = null;
    for (int i = 0; i < businessServiceVector.size(); i++) {
      String serviceName = ((BusinessService)businessServiceVector.elementAt(i)).getNameString();
      System.out.println ("[" + i + "] " + serviceName);
      if (serviceName.equals (SERVICE_NAME)) {
	businessService = (BusinessService)businessServiceVector.elementAt(i);
      }
    }

    System.out.println ("The service key for " + SERVICE_NAME + " is " +
                        businessService.getServiceKey());

    /* Get the binding templates- information on how to access the service */
    Vector bindingTemplateVector = 
       businessService.getBindingTemplates().getBindingTemplateVector();
    /* Find the access point for the first binding template */
    AccessPoint accessPoint = ((BindingTemplate)bindingTemplateVector.elementAt(0)).getAccessPoint();

    System.out.println ("The " + SERVICE_NAME + " can be accessed using "
                        + accessPoint.getURLType() 
			+ ". The access number/url/email is "
			+ accessPoint.getText());
  }
}
