package wrox.webservices.UDDIImpl;

import java.net.*;
import java.text.*;
import java.util.*;
import java.io.*;

import com.ibm.uddi.*;
import com.ibm.uddi.client.*;
import com.ibm.uddi.datatype.binding.*; 
import com.ibm.uddi.datatype.business.*; 
import com.ibm.uddi.datatype.service.*;
import com.ibm.uddi.datatype.tmodel.*;
import com.ibm.uddi.response.*;
import com.ibm.uddi.util.*;

import org.apache.soap.*; 
import org.apache.soap.rpc.*;

import org.apache.soap.transport.http.SOAPHTTPConnection;

/**
 * The FareClient class knows the tModel of the fare quoting 
 * service, and finds service based on that. It then invokes 
 * the getFare() method.
 *
 * This example is based on the premise that standards bodies
 * would define and publish tModels and individual  companies 
 * would provide implementations.
 */
public class FareClient {

  /* The flight iternary- we are flying from 
   * FROM_AIRPORT to TO_AIRPORT
   */
  private final static String FROM_AIRPORT = "SJC";
  private final static String TO_AIRPORT   = "TPE";

  public static void main (String args[]) {

    /* Process command line arguments */
    if (args.length != 1) {
      System.out.println 
         ("usage: java wrox.webserices.UDDIImpl.FareClient UUID:94B5B570-80B5-11D5-A76E-0004AC49CC1E
");
      System.exit (1);
    }

    /* Construct a UDDIProxy object- this represents a UDDI server 
     * and the actions that can be invoked against it.
     */
    UDDIProxy uddiProxy = new UDDIProxy ();
    /* Setting for the proxy server- in case you are behind one */
    
     SOAPHTTPConnection soapHTTPConnection = new SOAPHTTPConnection ();
    
     soapHTTPConnection.setProxyHost ("192.168.0.46");
     soapHTTPConnection.setProxyPort (80);
     uddiProxy.setTransport (soapHTTPConnection);
   

    /* Set the inquiry URL to the Test registry node at IBM */
    try {
      /* Had the business been published to the UDDI global registry
       * instead of the test registry, the inquiry URL could have been
       * set to any UDDI node (such as the one at Microsoft).
       */
      uddiProxy.setInquiryURL (UDDIOperatorURLs.IBM_INQUIRY_URL_TEST);
      System.out.println ("Inquiry URL : ["
                          + UDDIOperatorURLs.IBM_INQUIRY_URL_TEST + "]");
    } catch (MalformedURLException e) {
      System.err.println ("Malformed inquiry URL for the UDDI registry ["
                          + UDDIOperatorURLs.IBM_INQUIRY_URL_TEST + "]");
      e.printStackTrace ();
      System.exit (1); /* fatal error */
    }


    /* Assumption: There exists at least one tModel, Business, Service 
     *             that matches each of the searches. No error checking
     *             performed. We simply select the first found- in real
     *             life, there may be some business logic built in to
     *             select based on some other criteria.
     */

     /* Step 1: Locate tModel by well known tModel key
      *
      *      The OverviewURL of the tModel would point to the wsdl 
      *      description of the service.
      */
     
     String tModelKey = args[0]; /* passed as command line argument */

     /* Now that we have the required tModel key, drill down and get the 
       tModelDetail */

     TModelDetail tModelDetail = null;
     try {
         tModelDetail = uddiProxy.get_tModelDetail (tModelKey);
     } catch (Exception e) {
       System.err.println ("Exception finding tModel detail: " 
                           + e.getMessage ());
       e.printStackTrace ();
       System.exit (1); /* fatal error */
     }

     TModel tModel = (TModel)tModelDetail.getTModelVector().elementAt(0);
     String overviewURL = tModel.getOverviewDoc().getOverviewURLString();
     System.out.println ("OverviewURL of service is " + overviewURL);


     /* Step 2: Find businesses (businessEntity) that have a service that
      *      implements this tModel and and use some business logic to 
      *      select one from them.
      */
      Vector tmpTModelVector = new Vector();
      tmpTModelVector.addElement (tModelKey); /* found earlier */
      TModelBag tModelBag = new TModelBag (tmpTModelVector);
      BusinessList businessList = null;
      try {
        businessList = uddiProxy.find_business (tModelBag, null, 0);
      } catch (Exception e) {
        System.err.println ("Exception finding business entity: " 
                            + e.getMessage ());
        e.printStackTrace ();
        System.exit (1); /* fatal error */
      }

     Vector businessVector = businessList.getBusinessInfos().getBusinessInfoVector();
     /* Select one business out of the list- we select the first one */
     if (businessVector.isEmpty()) {
       System.err.println ("No Business found implemenenting tModel "
                           + tModelKey + ". Sorry!");
       System.exit (1);
     }
     String businessKey = ((BusinessInfo)businessVector.elementAt(0)).getBusinessKey ();

     /* Now that we have the required business key, we can drill down if
      * for additional information - the businessDetail structure. For
      * now, we dont need to do that.
      */

     /* Step 3: Locate the service (serviceInfo) within the business
      *      entity.
      *
      *      The AccessPoint would have the service URL - which in
      *      this case is the URL of the SOAP router/listener.
      */

      ServiceList serviceList = null;
      try {
        serviceList = uddiProxy.find_service (businessKey, tModelBag, null, 0);
      } catch (Exception e) {
        System.err.println ("Exception finding business service: " 
                            + e.getMessage ());
        e.printStackTrace ();
        System.exit (1); /* fatal error */
      }

     Vector serviceVector = serviceList.getServiceInfos().getServiceInfoVector();
     /* Taking the first business service matching this search */
     String serviceKey = ((ServiceInfo)serviceVector.elementAt(0)).getServiceKey();

     /* Now, drill down to get the ServiceDetail */

     ServiceDetail serviceDetail = null;
     try {
         serviceDetail = uddiProxy.get_serviceDetail (serviceKey);
     } catch (Exception e) {
       System.err.println ("Exception finding service detail: " 
                           + e.getMessage ());
       e.printStackTrace ();
       System.exit (1); /* fatal error */
     }

     BusinessService businessService = (BusinessService)serviceDetail.getBusinessServiceVector().elementAt(0);
     BindingTemplates bindingTemplates = businessService.getBindingTemplates();
     BindingTemplate bindingTemplate = (BindingTemplate) bindingTemplates.getBindingTemplateVector().elementAt(0);
     String accessPoint = bindingTemplate.getAccessPoint().getText();
     System.out.println ("accessPoint of service is " + accessPoint);

     /* Step 4: Bind and Invoke */

      /* Fare request for a flight from San Jose, California
       * to Taipei, Taiwan
       */
     float fare = (float) 0.0;
     try {
       FareService_ServiceProxy fareServiceProxy = 
                     new FareService_ServiceProxy();
       fareServiceProxy.setEndPoint (new URL(accessPoint));
       fare = fareServiceProxy.getFare (FROM_AIRPORT, TO_AIRPORT);
      }  catch (SOAPException e) {
        System.err.println ("Exception invoking method: "
                   + e.getFaultCode() + ": "
                   + e.getMessage ());
        e.printStackTrace ();
        System.exit (1); /* fatal error */
      } catch (MalformedURLException e) {
        System.err.println ("Malformed service URL ["
                          + accessPoint + "]");
        e.printStackTrace ();
        System.exit (1); /* fatal error */
      }

      System.out.println ("Requested fare from " + FROM_AIRPORT +
                          " to " + TO_AIRPORT +
                          " is $" + fare);
  }
}
