package wrox.webservices.UDDIImpl;

import java.net.*;
import java.util.*;
import java.io.*;

import com.ibm.uddi.*;
import com.ibm.uddi.client.*;
import com.ibm.uddi.response.*;
import com.ibm.uddi.datatype.*;
import com.ibm.uddi.datatype.tmodel.*;
import com.ibm.uddi.datatype.business.*; 
import com.ibm.uddi.util.*;

import org.apache.soap.SOAPException;
import org.apache.soap.transport.http.SOAPHTTPConnection;

/**
 * The SaveTModel class publishes a TModel description. 
 * A tModel acts as a unique signature for a service. Typically
 * standards bodies would publish tModels and individual companies
 * would write implementations conforming to it.
 */
public class SaveTModel {

  /* The username and password to interact with the UDDI registry */
  private final static String USERNAME = "wrox-chapter8";
  private final static String PASSWORD = "secret";
  private final static String WSDL_TMODEL_KEY 
    = "uuid:C1ACF26D-9672-4404-9D70-39B756E62AB4";

  /* The tModel information */
  private final static String TMODEL_NAME = "fare_service_tmodel";
  private final static String TMODEL_DESC = 
    "The fare service provides a programmatic API for getting airline fare information";
  private final static String WSDL_URL    = 
    "http://127.0.0.1:8080/wsdl/FareService_Service.wsdl";

  /* Helper method for constructing a vector with one element in it */
  private static Vector constructVector (Object object) {
    Vector vector = new Vector ();
    vector.addElement (object);
    return vector;
  }

  public static void main (String args[]) {

    /* Settings for HTTPS, SSL support */
    System.setProperty ("java.protocol.handler.pkgs", 
                        "com.ibm.net.ssl.internal.www.protocol");
    java.security.Security.addProvider (new com.ibm.jsse.JSSEProvider ());

    /* Construct a UDDIProxy object- this represents a UDDI server 
     * and the actions that can be invoked against it.
     */
    UDDIProxy uddiProxy = new UDDIProxy ();
    /* Setting for the proxy server- in case you are behind one */
    /*
     SOAPHTTPConnection soapHTTPConnection = new SOAPHTTPConnection ();
     soapHTTPConnection.setUserName ("proxy-username");
     soapHTTPConnection.setPassword ("proxy-password");
     soapHTTPConnection.setProxyHost ("proxyhostname");
     soapHTTPConnection.setProxyPort (8080);
     uddiProxy.setTransport (soapHTTPConnection);
    */


    /* Set the publish URL to the Test registry node at IBM */
    try {
      uddiProxy.setPublishURL (UDDIOperatorURLs.IBM_PUBLISH_URL_TEST);
      System.out.println ("Publishing tModel to " 
                          + UDDIOperatorURLs.IBM_PUBLISH_URL_TEST);
    } catch (MalformedURLException e) {
      System.err.println ("Malformed publish URL for the UDDI registry ["
                          + UDDIOperatorURLs.IBM_PUBLISH_URL_TEST + "]");
      e.printStackTrace ();
      System.exit (1); /* fatal error */
    }

    /* Get an authentication token */
    AuthToken authToken = null;
    try {
      authToken = uddiProxy.get_authToken(USERNAME, PASSWORD);
    } catch (UDDIException e) {
      System.err.println ("UDDI error getting authentication token: [" 
                          + e.getFaultCode() + "]" + e.getFaultString()
			 );
      DispositionReport dispositionReport = e.getDispositionReport ();
      System.err.println ("More error information : " 
                          + dispositionReport.getErrCode() + ": "
			  + dispositionReport.getErrInfoText());
      System.exit (1); /* fatal error */
    } catch (SOAPException e) {
      System.err.println ("SOAP error getting authentication token " 
                          + e.getFaultCode() + ": " + e.getMessage());
      System.exit (1); /* fatal error */
    }

    /* Construct the tModel - the empty tModel key should cause the UDDI 
     * registry to assign a (unique) key for the tModel.
     */
    TModel tModel = new TModel ("", TMODEL_NAME);
    tModel.setDefaultDescriptionString (TMODEL_DESC);

    OverviewDoc overviewDoc = new OverviewDoc ();
    overviewDoc.setOverviewURL (new OverviewURL (WSDL_URL));
    overviewDoc.setDefaultDescriptionString ("WSDL Document");
    tModel.setOverviewDoc (overviewDoc);

    KeyedReference keyedReference = new KeyedReference ();
    keyedReference.setKeyName ("uddi-org:types");
    keyedReference.setKeyValue ("wsdlSpec");
    keyedReference.setTModelKey (WSDL_TMODEL_KEY);

    CategoryBag categoryBag = new CategoryBag ();
    categoryBag.setKeyedReferenceVector (constructVector (keyedReference));
    tModel.setCategoryBag (categoryBag);

    /* Publish the tModel in the registry */
    TModelDetail tModelDetail = null;
    try {
      tModelDetail = uddiProxy.save_tModel (authToken.getAuthInfoString (),
                                            constructVector (tModel));
    } catch (UDDIException e) {
      System.err.println ("UDDI error saving tModel: [" 
                          + e.getFaultCode() + "]" + e.getFaultString()
			 );
      DispositionReport dispositionReport = e.getDispositionReport ();
      System.err.println ("More error information : " 
                          + dispositionReport.getErrCode() + ": "
			  + dispositionReport.getErrInfoText());
      System.exit (1); /* fatal error */
    } catch (SOAPException e) {
      System.err.println ("SOAP error saving tModel: " 
                          + e.getFaultCode() + ": " + e.getMessage());
      System.exit (1); /* fatal error */

    }

   /* Get the tModel key assigned for this tModel */
    Vector tModelVector = tModelDetail.getTModelVector();

   /* Since we saved only one tModel, the resulting tModel
    * detail should have only one tModel in it.
    */
    TModel thisTModel = (TModel) (tModelVector.elementAt(0));
    String tModelKey = thisTModel.getTModelKey ();

    System.out.println ("The " + TMODEL_NAME 
                      + " tModel has been successfully updated "
                      + " in the UDDI registry and assigned the tModel Key: "
                      + tModelKey);
  }
}
